try:
    from django.urls import re_path
except ImportError:
    from django.conf.urls import url as re_path

from . import views

app_name = 'perimeter'

urlpatterns = [
    re_path(r'^gateway/', views.gateway, name='gateway'),
]



from distutils.version import StrictVersion
from os import path

import django

DJANGO_VERSION = StrictVersion(django.get_version())

# set the django DEBUG option
DEBUG = True

PERIMETER_ENABLED = True

ROOT_URLCONF = 'test_app.urls'

# this isn't used, but Django likes having something here for running the tests
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'delme'
    }
}

# NB - this is good for local testing only
DEFAULT_FILE_STORAGE = 'django.core.files.storage.FileSystemStorage'
STATICFILES_STORAGE = 'django.contrib.staticfiles.storage.StaticFilesStorage'
STATIC_URL = '/static/'
STATIC_ROOT = path.abspath(path.join(path.dirname(__file__), '..', 'static'))

INSTALLED_APPS = (
    'django.contrib.contenttypes',
    'django.contrib.auth',
    'django.contrib.admin',
    'django.contrib.messages',
    'django.contrib.sessions',
    'django.contrib.staticfiles',
    'perimeter',
    'test_app',
)

ACTUAL_MIDDLEWARE_CLASSES = [
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'perimeter.middleware.PerimeterAccessMiddleware',
]

if DJANGO_VERSION < StrictVersion('1.10.0'):
    MIDDLEWARE_CLASSES = ACTUAL_MIDDLEWARE_CLASSES
else:
    MIDDLEWARE = ACTUAL_MIDDLEWARE_CLASSES

SECRET_KEY = "something really, really hard to guess goes here."

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'simple': {
            'format': '%(levelname)s %(message)s'
        },
    },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'simple'
        },
    },
    'loggers': {
        '': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'INFO',
        },
    }
}

PROJECT_DIR = path.abspath(path.join(path.dirname(__file__)))

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            path.join(PROJECT_DIR, 'templates'),
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    }
]



from django import forms
from django.core.exceptions import ValidationError

from .middleware import set_request_token
from .models import AccessToken


class TokenGatewayForm(forms.Form):

    """Form used to process a perimeter request."""

    token = forms.CharField(required=True, max_length=100)

    def clean_token(self):
        """Validate the token against existing tokens."""
        try:
            _token = self.cleaned_data.get('token')
            self.token = AccessToken.objects.get(token=_token)
            if self.token.is_valid:
                return _token
            if self.token.has_expired:
                raise ValidationError("Token has expired", code="expired")
            if not self.token.is_active:
                raise ValidationError("Token is inactive", code="invalid")
        except AccessToken.DoesNotExist:
            raise ValidationError("Token not found", code="invalid")

    def save(self, request):
        """Create a new AccessTokenUse object from the form."""
        assert getattr(self, 'token', None) is not None, "Form token attr is not set"
        set_request_token(request, self.token.token)
        return self.token.record(
            user_email=self.cleaned_data.get('email'),
            user_name=self.cleaned_data.get('name'),
            client_ip=request.META.get('REMOTE_ADDR', 'unknown'),
            client_user_agent=request.META.get('HTTP_USER_AGENT', 'unknown'),
        )


class UserGatewayForm(TokenGatewayForm):

    """Form used to process a perimeter request with user info."""

    email = forms.EmailField(required=True)
    name = forms.CharField(required=True, max_length=100)



from django.contrib import admin
try:
    from django.urls import re_path, include
except ImportError:
    from django.conf.urls import url as re_path, include
from django.views.generic import TemplateView

import perimeter.urls

admin.autodiscover()

urlpatterns = [
    re_path(r'^$', TemplateView.as_view(template_name='index.html'), name='homepage'),
    re_path(r'^admin/', admin.site.urls),
    re_path(r'^perimeter/', include(perimeter.urls)),
]
